<?php

namespace Drupal\campaign_kit\Controller;

use Drupal\campaign_kit_core\Entity\CampaignDonation;
use Drupal\Core\Controller\ControllerBase;
use Drupal\campaign_kit_core\Entity\Campaign;


use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Entity\EntityFormBuilderInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

//Added
use Drupal\Core\Routing\RouteProvider;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Database\Database;

// To create a link
use Drupal\Core\Url;
use Drupal\Core\Link;

/**
 * Class DonationGoalController.
 */
class DonationGoalController extends ControllerBase implements ContainerInjectionInterface {

  /**
   * The entity manager
   *
   * @var \Drupal\Core\Entity\EntityManagerInterface.
   */
  protected $entityManager;

  /**
   * The entity form builder
   *
   * @var \Drupal\Core\Entity\EntityManagerInterface.
   */
  protected $entityFormBuilder;

  /**
   * @var RouteProvider
   */
  protected $routeProvider;

  /**
   * DonationGoalController constructor.
   * @param EntityTypeManagerInterface $entityManager
   * @param EntityFormBuilderInterface $entityFormBuilder
   * @param RouteProvider $routeProvider
   */
  public function __construct(EntityTypeManagerInterface $entityManager, EntityFormBuilderInterface $entityFormBuilder, RouteProvider $routeProvider) {
    $this->entityManager = $entityManager;
    $this->entityFormBuilder = $entityFormBuilder;
    $this->routeProvider = $routeProvider;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    /** @noinspection PhpParamsInspection */
    return new static(
      $container->get('entity.manager'),
      $container->get('entity.form_builder'),
      $container->get('router.route_provider')
    );
  }

  /**
   * @return mixed
   */
  public function campaignKitPage() {
    $res['description']= [
      '#type' => 'markup',
    ];

    $linkCampaignList = '';
    $linkCampaignDonationList = '';
    $linkCampaignUpdateList = '';
    $linkCampaignTransactionList = '';
    $linkDonationList = '';

    // Campaign list
    $campaignListExist = count($this->routeProvider->getRoutesByNames(['entity.campaign.collection'])) === 1;
    if ($campaignListExist) {
      $campaignList = Url::fromRoute('entity.campaign.collection');
      $linkCampaignList = Link::fromTextAndUrl(t('Campaigns'), $campaignList);
    }

    // Campaign Donation list
    $campaignDonationListExist = count($this->routeProvider->getRoutesByNames(['entity.campaign_donation.collection'])) === 1;
    if ($campaignDonationListExist) {
      $campaignDonationList = Url::fromRoute('entity.campaign_donation.collection');
      $linkCampaignDonationList = Link::fromTextAndUrl(t('Campaign donations'), $campaignDonationList);
    }

    // Campaign Update list
    $campaignUpdateListExist = count($this->routeProvider->getRoutesByNames(['entity.campaign_update.collection'])) === 1;
    if ($campaignUpdateListExist) {
      $campaignUpdateList = Url::fromRoute('entity.campaign_update.collection');
      $linkCampaignUpdateList = Link::fromTextAndUrl(t('Campaign updates'), $campaignUpdateList);
    }

    // Campaign Transaction list
    $campaignTransactionListExist = count($this->routeProvider->getRoutesByNames(['entity.campaign_transaction.collection'])) === 1;
    if ($campaignTransactionListExist) {
      $campaignTransactionList = Url::fromRoute('entity.campaign_transaction.collection');
      $linkCampaignTransactionList = Link::fromTextAndUrl(t('Campaign transactions'), $campaignTransactionList);
    }

    // Donation list
    $donationListExist = count($this->routeProvider->getRoutesByNames(['entity.donation.collection'])) === 1;
    if ($donationListExist) {
      $donationList = Url::fromRoute('entity.donation.collection');
      $linkDonationList = Link::fromTextAndUrl(t('Donations'), $donationList);
    }

    $linksCampaignPages = array(
      $linkCampaignList,
      $linkCampaignDonationList,
      $linkCampaignUpdateList,
      $linkCampaignTransactionList,
      $linkDonationList,
    );
    $rows = [];
    foreach ($linksCampaignPages as $linksCampaignPage) {
      $rows[] = array(
        '#markup' => $linksCampaignPage
      );
    }

    // Build a render array which will be themed as a table with a pager.
    $res['pager_example'] = [
      '#rows' => $rows,
      '#header' => [t('Campaign links'),],
      '#type' => 'table',
      '#empty' => t('No content available.'),
    ];

    // Getting to Campaign Kit Configuration Form.
    $rowsPlugin = [];
    $campaignProcessorFormExist = count($this->routeProvider->getRoutesByNames(['campaign_kit.payment_processor_form'])) === 1;
    if ($campaignProcessorFormExist) {
      $campaignProcessorForm = Url::fromRoute('campaign_kit.payment_processor_form');
      $linkCampaignProcessorForm = Link::fromTextAndUrl(t('Campaign Kit Configuration Form'), $campaignProcessorForm);

      $plugins = array(
        $linkCampaignProcessorForm,
      );

      foreach ($plugins as $plugin) {
        $rowsPlugin[] = array(
          '#markup' => $plugin
        );
      }
    }

    // Build a render array which will be themed as a table with a pager.
    $res['plugin_table'] = [
      '#rows' => $rowsPlugin,
      '#header' => [t('Campaign Kit Configuration Form'),],
      '#type' => 'table',
      '#empty' => t('No content available.'),
    ];

    $res['plugin']= [
      '#type' => 'markup',
    ];

    // Getting link to Badge page(s)
    $rowsBadge = [];
    if (\Drupal::moduleHandler()->moduleExists('campaign_kit_demo')) {
      // Query demo Campaigns created by Campaign Kit Demo module.
      $ids = \Drupal::entityQuery('campaign')
        ->condition('user_id', '0', '=')
        ->range('0', '3')
        ->execute();
      $entries = Campaign::loadMultiple($ids);
      foreach ($entries as $entry => $value) {
        $campaignUrl = Url::fromRoute('campaign_kit.campaign_kit_controller_view_campaign_entity', array('campaign' => $value->id()));
        $campaignLink = Link::fromTextAndUrl($value->label(), $campaignUrl);
        $rowsBadge[] = array(
          '#markup' => $campaignLink
        );
      }
      // Get Badge Page link
      $badgePage = Url::fromRoute('campaign_kit.campaign_kit_controller_badge_page');
      $linkBadgePage = Link::fromTextAndUrl(t('Badges'), $badgePage);

      $badges = array(
        $linkBadgePage,
      );

      foreach ($badges as $badge) {
        $rowsBadge[] = array(
          '#markup' => $badge
        );
      }
    }

    // Build a render array which will be themed as a table with a pager.
    $res['badge_table'] = [
      '#rows' => $rowsBadge,
      '#header' => [t('Campaign Kit - Demonstration'),],
      '#type' => 'table',
      '#empty' =>  $rowsBadge? t('No content available.') : t('Please enable Campaign Kit Demonstration module to see examples.'),
    ];

    return $res;
  }

  /**
   * View Campaign entity.
   *
   * @param EntityInterface|NULL $campaign
   * @return array
   */
  public function viewCampaign(EntityInterface $campaign = NULL) {
    // Local variables.
    $campaignObject = array();
    $campaignId = $campaign->id();
    // Check if the entity exists.
    $ids = \Drupal::entityQuery('campaign')
      ->condition('id', $campaignId, '=')
      ->execute();

    if (is_numeric($campaignId)) {
      $entityCampaign = Campaign::load($campaignId);
      if ($entityCampaign->getCampaignType() == 'child') {
        $parent = $entityCampaign->getParent();
        $parentId = $parent->id();
        // Send this campaign to another controller.
        $campaignObject = $this->viewCampaignChild($parentId, $campaign);
      } else {
        if ($ids) {
          //View builder
          $entityType = 'campaign';
          $viewDisplay = 'campaign_kit';

          // Fields to render
          $fields = array(
            'title',
            'parent_id',
            'campaign_type',
            'header_image',
            'confirmation_email',
            'description',
            'short_description',
          );

          // Get the Campaign entity object.
          $entityCampaign = Campaign::load($campaignId);
          // Get the View for this Campaign entity object.
          $viewBuilder = \Drupal::entityTypeManager()->getViewBuilder($entityType);

          // Build the Campaign array to render.
          foreach ($fields as $field_name) {
            if ($entityCampaign->hasField($field_name) && $entityCampaign->access('view')) {
              $value = $entityCampaign->get($field_name);
              $output = $viewBuilder->viewField($value, $viewDisplay);
              $output['#cache']['tags'] = $entityCampaign->getCacheTags();
              $campaignObject[$field_name] = $output;
            }
          }
          // Get the campaign entity.
          // Build the Campaign object.
          $campaignObject['id'] = $entityCampaign->getCampaignID();
          $campaignObject['one_time_donation_goal'] = (int)$entityCampaign->getGoal();
          $campaignObject['monthly_donation_goal'] = (int)$entityCampaign->getGoalSustaining();
          $campaignObject['campaign_status'] = $entityCampaign->getCampaignStatus();
          $campaignObject['facebook_url'] = $entityCampaign->getFacebookPath();
          $campaignObject['twitter_url'] = $entityCampaign->getTwitterPath();
          $campaignObject['google_url'] = $entityCampaign->getGooglePath();
          $campaignObject['donation_frequency_allowed'] = $entityCampaign->getFrequencyAllowed();
          $campaignObject['campaign_type'] = $entityCampaign->getCampaignType();
          $campaignObject['daysLeft'] = $entityCampaign->getDaysLeft();
          $campaignObject['goal'] = $entityCampaign->getGoal();
          $campaignObject['author_name'] = $entityCampaign->getOwnerName();

          // Get Campaign's money information.
          $campaignObject['numSupporters'] = $entityCampaign->getNumberSupporters();
          $campaignObject['goalSaved'] = $entityCampaign->getRaisedAmount();
          $campaignObject['percentage'] = $entityCampaign->getPercentage();
          // Get currency from plugin.
          $config = \Drupal::config('campaign_kit.settings');
          $clientPlugin =  $config->get('campaign_kit');
          $campaignObject['currency'] = $clientPlugin['currency'];

          // Create links for Campaign Page.
          // I want to participate link destination.
          $destination = Url::fromRoute('campaign_kit.campaign_kit_controller_add_child_page', array('campaignParentId' => $campaignObject['id']));

          $loginUrl = Url::fromRoute(
            'user.login',
            array(),
            array(
              'query' => array('destination' => $destination->getInternalPath())
            )
          );

          if (\Drupal::currentUser()->isAnonymous()) {
            $linkParticipate = Link::fromTextAndUrl(t('I Want to Join this Campaign!'), $loginUrl);
          }
          else {
            $linkParticipate = Link::fromTextAndUrl(t('I Want to Join this Campaign!'), $destination);
          }

          $campaignObject['participate_link'] = $linkParticipate->toRenderable();
          $campaignObject['participate_link']['#attributes'] = array('class' => array('donate-button'));

          // Donate link.
          $urlDonation = Url::fromRoute('campaign_kit.donate_form_campaign', array('campaignId' => $campaignObject['id']));
          $linkDonation = Link::fromTextAndUrl(t('Donate Now'), $urlDonation);
          $campaignObject['donate_link'] = $linkDonation->toRenderable();
          $campaignObject['donate_link']['#attributes'] = array('class' => array('donate-button'));

          // Create social network links.
          if ($entityCampaign->getFacebookPath()) {
            $urlFacebook = Url::fromUri($entityCampaign->getFacebookPath());
            $linkFacebook = Link::fromTextAndUrl(t('Share on Facebook'), $urlFacebook);
            $campaignObject['facebook_link'] = $linkFacebook->toRenderable();
            $campaignObject['facebook_link']['#attributes'] = array('class' => array('facebook-icon'), 'target' => '_blank', 'title' => t('Share on Facebook'));
          }

          if ($entityCampaign->getTwitterPath()) {
            $urlTwitter = Url::fromUri($entityCampaign->getTwitterPath());
            $linkTwitter = Link::fromTextAndUrl(t('Share on Twitter'), $urlTwitter);
            $campaignObject['twitter_link'] = $linkTwitter->toRenderable();
            $campaignObject['twitter_link']['#attributes'] = array('class' => array('twitter-icon'), 'target' => '_blank', 'title' => t('Share on Twitter'));
          }

          if($entityCampaign->getGooglePath()) {
            $urlGoogle = Url::fromUri($entityCampaign->getGooglePath());
            $linkGoogle = Link::fromTextAndUrl(t('Share on Google+'), $urlGoogle);
            $campaignObject['google_link'] = $linkGoogle->toRenderable();
            $campaignObject['google_link']['#attributes'] = array('class' => array('google-icon'), 'target' => '_blank', 'title' => t('Share on Google +'));
          }

          // Get the alias for the current campaign.
          $isItEnabled = \Drupal::moduleHandler()->moduleExists('pathauto');
          $routName = \Drupal::service('path.current')->getPath();
          if ($isItEnabled) {
            $result = \Drupal::service('path.alias_manager')->getAliasByPath($routName);
            if ($result) {
              $bodyLink = $result;
            } else {
              $bodyLink = $routName;
            }
          } else {
            $bodyLink = $routName;
          }
          $hostHttp = \Drupal::request()->getSchemeAndHttpHost();
          $bodyLink = $hostHttp.$bodyLink;

          // Create link for email a campaign.
          $subject = t('Check out this fantastic campaign!');
          $body = t('Have a look at this campaign and please help if you are inspired to!');
          $urlFriend= Url::fromUri('mailto:?subject='.$subject.'&body='.$body.' '.$bodyLink);
          $linkFriend = Link::fromTextAndUrl(t('Email a friend'), $urlFriend);
          $campaignObject['email_friend_link'] = $linkFriend->toRenderable();
          $campaignObject['email_friend_link']['#attributes'] = array('class' => array('email-icon'), 'title' => t('Email a friend'));

          // Create Subscribe link
          $urlSubscribe= Url::fromUri('https://www.facebook.com/');
          $linkSubscribe = Link::fromTextAndUrl(t('Subscribe to Updates'), $urlSubscribe);
          $campaignObject['subscribe_link'] = $linkSubscribe->toRenderable();
          $campaignObject['subscribe_link']['#attributes'] = array('class' => array('subscribe-button'), 'title' => t('Subscribe to Updates'));

          // Create Continue to homepage link
          $urlHome = Url::fromRoute('<front>');
          $linkHome = Link::fromTextAndUrl(t('Continue to homepage'), $urlHome);
          $campaignObject['home_page_link'] = $linkHome->toRenderable();
          $campaignObject['home_page_link']['#attributes'] = array('class' => array('default-button'), 'title' => t('Continue to homepage'));

          // Get views and add them to Campaign array
          $renderSupporters = views_embed_view('campaign_kit_supporters', array('display_id' => 'block_1', 'check_access' => TRUE));
          $renderCampaignUpdates = views_embed_view('campaign_kit_updates', array('display_id' => 'block_1', 'check_access' => TRUE));

          $campaignObject['supportersView'] = $renderSupporters;
          $campaignObject['updatesView'] = $renderCampaignUpdates;

        } else {
          drupal_set_message(t("Unable to find campaign."), 'error');
        }
      }
    }

    $res = array(
      '#theme' => 'campaign--display',
      '#campaign' => $campaignObject,
    );

    return $res;
  }

  /**
   * View Campaign Child entity.
   *
   * @param $campaignParentId
   * @param EntityInterface $campaign
   * @return array
   */
  public function viewCampaignChild($campaignParentId, EntityInterface $campaign) {
    //Local variables
    $campaignObject = array();
    $campaignChildId = $campaign->id();

    // Check if the Campaign parent entity exists
    $idParent = \Drupal::entityQuery('campaign')
      ->condition('id', $campaignParentId, '=')
      ->condition('campaign_type', 'parent', '=')
      ->execute();
    if ($idParent) {
      // Check if the Campaign child entity exists
      $idChild = \Drupal::entityQuery('campaign')
        ->condition('id', $campaignChildId, '=')
        ->condition('campaign_type', 'child', '=')
        ->execute();

      if ($idChild) {
        // View builder
        $entityType = 'campaign';
        $viewDisplay = 'campaign_kit';

        // Fields to render
        $fields = array(
          'title',
          'parent_id',
          'campaign_type',
          'header_image',
          'confirmation_email',
          'description',
          'short_description',
        );

        // Get the Campaign entity object
        $entityCampaign = Campaign::load($campaignChildId);
        // Get the View for this Campaign entity object
        $viewBuilder = \Drupal::entityTypeManager()->getViewBuilder($entityType);

        // Build the Campaign array to render
        foreach ($fields as $field_name) {
          if ($entityCampaign->hasField($field_name) && $entityCampaign->access('view')) {
            $value = $entityCampaign->get($field_name);
            $output = $viewBuilder->viewField($value, $viewDisplay);
            $output['#cache']['tags'] = $entityCampaign->getCacheTags();
            $campaignObject[$field_name] = $output;
          }
        }
        // Get the campaign entity
        // Build the Campaign object
        $campaignObject['id'] = $entityCampaign->getCampaignID();
        $campaignObject['one_time_donation_goal'] = (int)$entityCampaign->getGoal();
        $campaignObject['monthly_donation_goal'] = (int)$entityCampaign->getGoalSustaining();
        $campaignObject['campaign_status'] = $entityCampaign->getCampaignStatus();
        $campaignObject['facebook_url'] = $entityCampaign->getFacebookPath();
        $campaignObject['twitter_url'] = $entityCampaign->getTwitterPath();
        $campaignObject['google_url'] = $entityCampaign->getGooglePath();
        $campaignObject['donation_frequency_allowed'] = $entityCampaign->getFrequencyAllowed();
        $campaignObject['campaign_type'] = $entityCampaign->getCampaignType();
        $campaignObject['daysLeft'] = $entityCampaign->getDaysLeft();
        $campaignObject['goal'] = (int)$entityCampaign->getGoal();
        $campaignObject['author_name'] = $entityCampaign->getOwnerName();

        // Get Campaign's money information
        $campaignObject['numSupporters'] = $entityCampaign->getNumberSupporters();
        $campaignObject['goalSaved'] = $entityCampaign->getRaisedAmount();
        $campaignObject['percentage'] = $entityCampaign->getPercentage();
        // Get currency from plugin
        $config = \Drupal::config('campaign_kit.settings');
        $clientPlugin =  $config->get('campaign_kit');
        $campaignObject['currency'] = $clientPlugin['currency'];

        // Create links for Campaign Page
        // Donate link
        $urlDonation = Url::fromRoute('campaign_kit.donate_form_campaign_child', array('campaignId' => $campaignParentId, 'campaignChildId' => $campaignObject['id']));
        $linkDonation = Link::fromTextAndUrl(t('Donate Now'), $urlDonation);
        $campaignObject['donate_link'] = $linkDonation->toRenderable();
        $campaignObject['donate_link']['#attributes'] = array('class' => array('donate-button'));

        // Create social network links
        if ($entityCampaign->getFacebookPath()) {
          $urlFacebook = Url::fromUri($entityCampaign->getFacebookPath());
          $linkFacebook = Link::fromTextAndUrl(t('Share on Facebook'), $urlFacebook);
          $campaignObject['facebook_link'] = $linkFacebook->toRenderable();
          $campaignObject['facebook_link']['#attributes'] = array('class' => array('facebook-icon'), 'target' => '_blank', 'title' => t('Share on Facebook'));
        }

        if ($entityCampaign->getTwitterPath()) {
          $urlTwitter = Url::fromUri($entityCampaign->getTwitterPath());
          $linkTwitter = Link::fromTextAndUrl(t('Share on Twitter'), $urlTwitter);
          $campaignObject['twitter_link'] = $linkTwitter->toRenderable();
          $campaignObject['twitter_link']['#attributes'] = array('class' => array('twitter-icon'), 'target' => '_blank', 'title' => t('Share on Twitter'));
        }

        if ($entityCampaign->getGooglePath()) {
          $urlGoogle = Url::fromUri($entityCampaign->getGooglePath());
          $linkGoogle = Link::fromTextAndUrl(t('Share on Google+'), $urlGoogle);
          $campaignObject['google_link'] = $linkGoogle->toRenderable();
          $campaignObject['google_link']['#attributes'] = array('class' => array('google-icon'), 'target' => '_blank', 'title' => t('Share on Google +'));
        }

        // Get the alias for the current campaign.
        $isItEnabled = \Drupal::moduleHandler()->moduleExists('pathauto');
        $routName = \Drupal::service('path.current')->getPath();
        if ($isItEnabled) {
          $result = \Drupal::service('path.alias_manager')->getAliasByPath($routName);
          if ($result) {
            $bodyLink = $result;
          } else {
            $bodyLink = $routName;
          }
        } else {
          $bodyLink = $routName;
        }
        $hostHttp = \Drupal::request()->getSchemeAndHttpHost();
        $bodyLink = $hostHttp.$bodyLink;

        // Create link for email a campaign.
        $subject = t('Check out this fantastic campaign!');
        $body = t('Have a look at this campaign and please help if you are inspired to!');
        $urlFriend= Url::fromUri('mailto:?subject='.$subject.'&body='.$body.' '.$bodyLink);
        $linkFriend = Link::fromTextAndUrl(t('Email a friend'), $urlFriend);
        $campaignObject['email_friend_link'] = $linkFriend->toRenderable();
        $campaignObject['email_friend_link']['#attributes'] = array('class' => array('email-icon'), 'title' => t('Email a friend'));

        // Create Subscribe link
        $urlSubscribe= Url::fromUri('https://www.facebook.com/');
        $linkSubscribe = Link::fromTextAndUrl(t('Subscribe to Updates'), $urlSubscribe);
        $campaignObject['subscribe_link'] = $linkSubscribe->toRenderable();
        $campaignObject['subscribe_link']['#attributes'] = array('class' => array('subscribe-button'), 'title' => t('Subscribe to Updates'));

        // Create Continue to homepage link
        $urlHome = Url::fromRoute('<front>');
        $linkHome = Link::fromTextAndUrl(t('Continue to homepage'), $urlHome);
        $campaignObject['home_page_link'] = $linkHome->toRenderable();
        $campaignObject['home_page_link']['#attributes'] = array('class' => array('subscribe-icon'), 'title' => t('Continue to homepage'));

        // Render Supporters and Campaign Update views
        $renderSupporters = views_embed_view('campaign_kit_supporters', array('display_id' => 'block_1', 'check_access' => TRUE));
        $renderCampaignUpdates = views_embed_view('campaign_kit_updates', array('display_id' => 'block_1', 'check_access' => TRUE));
        $campaignObject['supportersView'] = $renderSupporters;
        $campaignObject['updatesView'] = $renderCampaignUpdates;

      } else {
        drupal_set_message(t("Unable to find campaign."), 'error');
      }
    } else {
      drupal_set_message(t("Unable to find campaign."), 'error');
    }

    /*
     * We are using only one template to render any Campaign entity.
    // Response
    $res = array(
      '#theme' => 'campaign--display',
      '#campaignChild' => $campaignObject,
    );

    return $res;
    */
    return $campaignObject;
  }

  /**
   * Badge page.
   *
   * @return array
   *   Return Template for Badge page.
   */
  public function badge() {
    // Let's check if our demo module is enabled
    $isItEnabled = \Drupal::moduleHandler()->moduleExists('campaign_kit_demo');

    $renderBadge = views_embed_view('campaign_kit_badges', array('display_id' => 'block_1', 'check_access' => TRUE));

    $res = array(
      '#theme'=> 'campaign-badge',
      '#enable'=> $isItEnabled,
      '#badge'=> $renderBadge,
    );

    return $res;
  }

  /**
   * Create Child Campaign page.
   *
   * @param $campaignParentId
   *
   * @return array.
   * @internal param $idCampaign
   *
   */
  public function addChild($campaignParentId) {
    $campaign = $this->entityManager->getStorage('campaign') ->create(array('type' => 'child'));
    $childCreateForm = $this->entityFormBuilder->getForm($campaign, 'child');

    $res = array(
      '#theme' => 'campaign--child--add',
      '#child_create_form' => $childCreateForm,
    );
    return $res;
  }

  /**
   * Thank you with Campaign ID page.
   *
   * @param EntityInterface $campaign
   * @param $campaignDonationId
   * @return array
   */
  public function thankYouId(EntityInterface $campaign, $campaignDonationId) {
    //Local variables
    $campaignObject = array();
    $campaignId = $campaign->id();
    $ids = '';
    if (is_numeric($campaignId)){
      // Check if the entity exists
      $ids = \Drupal::entityQuery('campaign')
        ->condition('id', $campaignId, '=')
        ->execute();
    } else {
      drupal_set_message(t("Unable to find campaign."), 'error');
    }

    if ($ids) {
      // View builder
      $entityType = 'campaign';
      $viewDisplay = 'campaign_kit';

      // Fields to render
      $fields = array(
        'title',
        'parent_id',
        'campaign_type',
        'header_image',
        'mail',
        'thank_you_message',
        'donation_page_title',
        'donation_page_description',
      );

      // Get the Campaign entity object
      $entityCampaign = Campaign::load($campaignId);
      // Get the View for this Campaign entity object
      $viewBuilder = \Drupal::entityTypeManager()->getViewBuilder($entityType);

      // Build the array to render
      foreach ($fields as $field_name) {
        if ($entityCampaign->hasField($field_name) && $entityCampaign->access('view')) {
          $value = $entityCampaign->get($field_name);
          $output = $viewBuilder->viewField($value, $viewDisplay);
          $output['#cache']['tags'] = $entityCampaign->getCacheTags();
          $campaignObject[$field_name] = $output;
        }
      }
      // Check if it is a parent or child
      if ($entityCampaign->get('campaign_type')->value == 'child') {
        $parent = $entityCampaign->getParent();
        // Back to campaign link
        $urlBackToCampaign = Url::fromRoute('campaign_kit.campaign_kit_controller_view_campaign_child_entity', array(
          'campaignParentId' => $parent->id->value,
          'campaign' => $campaignId,
        ));
        $linkBackToCampaign = Link::fromTextAndUrl(t('Back to Campaign Page!'), $urlBackToCampaign);
        $campaignObject['back_to_campaign_link'] = $linkBackToCampaign->toRenderable();
        $campaignObject['back_to_campaign_link']['#attributes'] = array('class' => array('donate-button', 'campaign-default--button'));

      } else {
        // Back to campaign link
        $urlBackToCampaign = Url::fromRoute('campaign_kit.campaign_kit_controller_view_campaign_entity', array(
          'campaign' => $campaignId
        ));
        $linkBackToCampaign = Link::fromTextAndUrl(t('Back to Campaign Page!'), $urlBackToCampaign);
        $campaignObject['back_to_campaign_link'] = $linkBackToCampaign->toRenderable();
        $campaignObject['back_to_campaign_link']['#attributes'] = array('class' => array('donate-button', 'campaign-default--button'));
      }

      // Query Campaign Donation fields.
      $idCampaignDonation = \Drupal::entityQuery('campaign_donation')
        ->condition('id', $campaignDonationId, '=')
        ->condition('campaign_id', $campaignId, '=')
        ->execute();

      if ($idCampaignDonation){
        // Load Campaign Donation entity
        $entityCampaignDonation = CampaignDonation::load(reset($idCampaignDonation));

        //View builder
        $entityCampaignDonationType = 'campaign_donation';
        $viewModeCampaignDonation = 'default';

        // Fields to render
        $fieldsCampaignDonation = array(
          'payment_processor',
          'recipient_message',
        );

        $viewBuilderCampaignDonation = \Drupal::entityTypeManager()->getViewBuilder($entityCampaignDonationType);

        foreach ($fieldsCampaignDonation as $field_name) {
          if ($entityCampaignDonation->hasField($field_name) && $entityCampaignDonation->access('view')) {
            $value2 = $entityCampaignDonation->get($field_name);
            $output2 = $viewBuilderCampaignDonation->viewField($value2, $viewModeCampaignDonation);
            $output2['#cache']['tags'] = $entityCampaignDonation->getCacheTags();
            $campaignObject[$field_name] = $output2;
          }
        }

        if ($entityCampaignDonation->get('dedicate_donation')->value == '1'){
          $campaignObject['dedicate_donation'] = true;
        } else {
          $campaignObject['dedicate_donation'] = false;
        }
        // Getting just values
        $campaignObject['dedicate_type'] = $entityCampaignDonation->get('dedicate_type')->value;
        $campaignObject['honoree_first_name'] = $entityCampaignDonation->get('honoree_first_name')->value;
        $campaignObject['honoree_last_name'] = $entityCampaignDonation->get('honoree_last_name')->value;
        $campaignObject['recipient_mail'] = $entityCampaignDonation->get('recipient_mail')->value;
        $campaignObject['recipient_first_name'] = $entityCampaignDonation->get('recipient_first_name')->value;
        $campaignObject['recipient_last_name'] = $entityCampaignDonation->get('recipient_last_name')->value;

      } else {
        drupal_set_message(t("There are no donations for this campaign yet."), 'error');
      }

    } else {
      drupal_set_message(t("Unable to find campaign."), 'error');
    }
    $res = array(
      '#theme'=> 'campaign--thank-you',
      '#campaign' => $campaignObject,
    );

    return $res;
  }

  /**
   * @param $campaignParentId
   * @param $campaignChildId
   *
   * @param $campaignDonationId
   *
   * @return array
   */
  public function thankYouChild($campaignParentId, $campaignChildId, $campaignDonationId) {
    $campaign = array();
    $idParent = '';
    if (is_numeric($campaignParentId)){
      //Check if the entity exists
      $idParent = \Drupal::entityQuery('campaign')
        ->condition('id', $campaignParentId, '=')
        ->execute();
    } else {
      drupal_set_message(t("Unable to find campaign."), 'error');
    }

    if ($idParent) {
      //Check if the Campaign child entity exists
      $idChild = \Drupal::entityQuery('campaign')
        ->condition('id', $campaignChildId, '=')
        ->condition('campaign_type', 'child', '=')
        ->execute();

      if ($idChild) {
        //View builder
        $entityType = 'campaign';
        $viewDisplay = 'campaign_kit';

        // Fields to render
        $fields = [
          'title',
          'parent_id',
          'campaign_type',
          'header_image',
          'confirmation_email',
          'thank_you_message',
          'donation_page_title',
          'donation_page_description',
        ];

        // Get the Campaign entity object
        $entityCampaign = Campaign::load($campaignChildId);
        // Get the View for this Campaign entity object
        $viewBuilder = \Drupal::entityTypeManager()
          ->getViewBuilder($entityType);

        // Build the array to render
        foreach ($fields as $field_name) {
          if ($entityCampaign->hasField($field_name) && $entityCampaign->access('view')) {
            $value = $entityCampaign->get($field_name);
            $output = $viewBuilder->viewField($value, $viewDisplay);
            //$output['#cache']['tags'] = $entityCampaign->getCacheTags();
            $campaign[$field_name] = $output;
          }
        }

        // Check if it is a parent or child
        if ($entityCampaign->get('campaign_type')->value == 'child') {
          // Back to campaign link
          $urlBackToCampaign = Url::fromRoute('campaign_kit.campaign_kit_controller_view_campaign_entity', array(
            'campaign' => $campaignChildId
          ));
          $linkBackToCampaign = Link::fromTextAndUrl(t('Back to Campaign Page!'), $urlBackToCampaign);
          $campaign['back_to_campaign_link'] = $linkBackToCampaign->toRenderable();
          $campaign['back_to_campaign_link']['#attributes'] = [
            'class' => [
              'donate-button',
              'campaign-default--button'
            ]
          ];
        }

        // Query Campaign Donation fields.
        $idCampaignDonation = \Drupal::entityQuery('campaign_donation')
          //->condition('first_name', 'Andre CK', '=')
          ->condition('id', $campaignDonationId, '=')
          ->condition('campaign_id', $campaignChildId, '=')
          ->execute();

        if ($idCampaignDonation) {
          // Load Campaign Donation entity
          $entityCampaignDonation = CampaignDonation::load(reset($idCampaignDonation));

          //View builder
          $entityCampaignDonationType = 'campaign_donation';
          $viewModeCampaignDonation = 'default';

          // Fields to render
          $fieldsCampaignDonation = [
            'payment_processor',
            'recipient_message',
          ];

          $viewBuilderCampaignDonation = \Drupal::entityTypeManager()
            ->getViewBuilder($entityCampaignDonationType);

          foreach ($fieldsCampaignDonation as $field_name) {
            if ($entityCampaignDonation->hasField($field_name) && $entityCampaignDonation->access('view')) {
              $value2 = $entityCampaignDonation->get($field_name);
              $output2 = $viewBuilderCampaignDonation->viewField($value2, $viewModeCampaignDonation);
              $output2['#cache']['tags'] = $entityCampaignDonation->getCacheTags();
              $campaign[$field_name] = $output2;
            }
          }

          // Set True or False for 'dedicate_donation' field
          if ($entityCampaignDonation->get('dedicate_donation')->value == '1') {
            $campaign['dedicate_donation'] = TRUE;
          }
          else {
            $campaign['dedicate_donation'] = FALSE;
          }
          // TODO: Check recipient message
          $campaign['dedicate_type'] = $entityCampaignDonation->get('dedicate_type')->value;
          $campaign['honoree_first_name'] = $entityCampaignDonation->get('honoree_first_name')->value;
          $campaign['honoree_last_name'] = $entityCampaignDonation->get('honoree_last_name')->value;
          $campaign['recipient_mail'] = $entityCampaignDonation->get('recipient_mail')->value;
          $campaign['recipient_first_name'] = $entityCampaignDonation->get('recipient_first_name')->value;
          $campaign['recipient_last_name'] = $entityCampaignDonation->get('recipient_last_name')->value;
        } else {
          drupal_set_message(t("There are no donations for this campaign yet."), 'error');
        }

      }
      else {
        drupal_set_message(t("Unable to find campaign."), 'error');
      }
    } else {
      drupal_set_message(t("Unable to find campaign."), 'error');
    }

    //Response
    $res = array(
      '#theme' => 'campaign--child--thank-you',
      '#campaignChild' => $campaign,
    );

    return $res;
  }

  /**
   * Returns Campaign title.
   *
   * @param EntityInterface $campaign
   * @return string
   */
  public function getCampaignTitle(EntityInterface $campaign) {
    $campaignId = $campaign->id();
    //Check if the entity exists
    $ids = \Drupal::entityQuery('campaign')
      ->condition('id', $campaignId, '=')
      ->execute();

    if($ids) {
      $entityCampaign = Campaign::load($campaignId);

      return $entityCampaign->get('title')->value;
    } else {
      return '';
    }
  }
}
