<?php

namespace Drupal\campaign_kit\Form;

use Drupal\campaign_kit_core\Entity\Campaign;
use Drupal\campaign_kit_core\Entity\CampaignTransaction;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;

// Added
use Drupal\campaign_kit_core\Entity\CampaignDonation;
use Drupal\Core\Locale\CountryManager;
use Drupal\payment_stripe\Event\StripeWebhookChargeFailed;
use Drupal\payment\Entity\Payment;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;
use Drupal\payment_stripe\Event\StripeEvents;
use Drupal\payment_stripe\Event\StripeWebhookChargeSucceeded;
use Drupal\campaign_kit\Campaign\CampaignPayment;

/**
 * Class DonateForm.
 */
class DonateForm extends FormBase {

  /**
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected $event_dispatcher;

  /**
   * @var CampaignPayment
   */
  protected $campaign_service;

  /**
   * Class constructor.
   */
  public function __construct(EventDispatcherInterface $event_dispatcher, CampaignPayment $campaign_service) {
    $this->event_dispatcher = $event_dispatcher;
    $this->campaign_service = $campaign_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    // Instantiates this form class.
    return new static(
    // Load the service required to construct this class.
      $container->get('event_dispatcher'),
      $container->get('campaign_kit.campaign_payment')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'donate_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $campaignId = NULL, $campaignChildId = NULL) {
    $campaignStatus = '';
    // Check if the Campaign is 'standalone or parent or child'
    if($campaignId && $campaignChildId) {
      $form['campaign_type'] = array(
        '#type' => 'hidden',
        '#value' => 'child',
      );
      $campaignChild = Campaign::load($campaignChildId);
      $campaignParentId = $campaignChild->getParent();
      $form['parent_id'] = array(
        '#type' => 'hidden',
        '#value' => $campaignParentId->id(),
      );

      $query = db_select('campaign_field_data', 'c')
        ->condition('id', $campaignChildId, '=')
        ->fields('c', array('campaign_status'));

      $result = $query->execute();
      foreach ($result as $record) {
        $campaignStatus = $record->campaign_status;
      }
      $currentCampaign = $campaignChildId;

    } else {
      $form['campaign_type'] = array(
        '#type' => 'hidden',
        '#value' => 'standalone',
      );

      $query = db_select('campaign_field_data', 'c')
        ->condition('id', $campaignId, '=')
        ->fields('c', array('campaign_status'));

      $result = $query->execute();
      foreach ($result as $record) {
        $campaignStatus = $record->campaign_status;
      }
      $currentCampaign = $campaignId;
    }

    // Getting country list
    $countryList = CountryManager::getStandardList();
    $countries = array();
    foreach ($countryList as $key => $value) {
      $countries[$key] = (string)$value;
    }

    // Load Campaign
    $currentCampaignLoaded = Campaign::load($currentCampaign);

    if ($campaignStatus == "open") {

      $form['campaign_status'] = array(
        '#type' => 'hidden',
        '#value' => $currentCampaign,
      );

      // Get currency from plugin
      // TODO: Use the new name for the configuration.
      $config = \Drupal::config('campaign_kit.settings');
      $clientPlugin =  $config->get('campaign_kit');
      $clientPlugin['currency'];

      // frequency
      $frequency = [];
      $campaignFrequency = $currentCampaignLoaded->getFrequencyAllowed();
      switch ($campaignFrequency){
        case 'onetime':
          $frequency = array(
            'onetime' => $this->t('One-time'),
          );
          break;
        case 'monthly':
          $frequency = array(
            'monthly' => $this->t('Monthly'),
          );
          break;
        case 'both':
          $frequency = array(
            'onetime' => $this->t('One-time'),
            'monthly' => $this->t('Monthly'),
          );
          break;
      }

      // Donation fields
      $form['amount'] = [
        '#type' => 'fieldset',
        '#title' => $this->t('Select amount'),
      ];

      // TODO: Ask Andre about this
      $form['amount']['lc'] = array(
        '#type' => 'hidden',
        '#value' => 'US',
      );

      $form['amount']['currency_code'] = array(
        '#type' => 'hidden',
        '#value' => $clientPlugin['currency'],
      );

      $form['amount']['item_number'] = array(
        '#type' => 'hidden',
        '#value' => $currentCampaignLoaded->id(),
      );

      $form['amount']['test_item'] = array(
        '#type' => 'hidden',
        '#value' => $currentCampaignLoaded->label(),
      );

      $form['amount']['txn_id'] = array(
        '#type' => 'hidden',
        '#value' => 'txn_id',
      );

      $form['amount']['settings']['donation'] = array(
        '#type' => 'radios',
        '#default_value' => 'other',
        '#options' => array(
          '25.00' => $this->t('$25'),
          '50.00' => $this->t('$50'),
          '100.00' => $this->t('$100'),
          '500.00' => $this->t('$500'),
          'other' => $this->t('Other'),
        ),
        '#attributes' => array(
          'onclick' => 'getdonationamount()'),
      );

      $form['amount']['amount'] = [
        '#type' => 'number',
        '#prefix' => '<div class="donation-data-amount">',
        '#title' => t('Your donation'),
        '#required' => TRUE,
        '#field_suffix' => $clientPlugin['currency'],
        '#attributes' => [
          'min' => '0',
          'max' => '999999',
        ],
      ];

      // TODO: Update this with plugin name of new config
      $form['amount']['recurrence']['time'] = array(
        '#type' => 'radios',
        '#default_value' => $campaignFrequency,
        '#options' => $frequency,
        '#attributes' => array(
          'required' => TRUE
        ),
        '#suffix' => '</div>',
      );

      // CheckBoxes.
      $form['amount']['dedication']['dedicate_donation'] = [
        '#type' => 'checkboxes',
        '#options' => ['1' => t('Make my donation in honor or in memory of someone')],
        '#attributes' => array(
          'onclick' => 'hasdedication()'
        ),
      ];

      $form['dedication-information'] = [
        '#type' => 'fieldset',
      ];

      $form['dedication-information']['dedication-type']['dedicate_type'] = array(
        '#type' => 'radios',
        '#default_value' => 'in_honor_of',
        '#options' => array(
          'in_honor_of' => $this->t('In honor of...'),
          'in_memory_of' => $this->t('In memory of...'),
        ),
      );

      $form['dedication-information']['honoree_first_name'] = array(
        '#type' => 'textfield',
        '#prefix' => '<div class="data-dedication-info">',
        '#title' => t("Honoree's Name "),
        '#maxlength' => '29',
        '#attributes' => array('placeholder' => t('First Name')),
      );

      $form['dedication-information']['honoree_last_name'] = array(
        '#type' => 'textfield',
        '#suffix' => '</div>',
        '#title' => t("Honoree's Last Name "),
        '#maxlength' => '29',
        '#attributes' => array('placeholder' => t('Last Name')),
      );

      // Email Recipient’s Info.
      $form['dedication-information']['recipient_mail'] = [
        '#type' => 'email',
        '#title' => t("Recipient’s Email"),
        '#maxlength' => '60',
        '#description' => 'Optionally send a notification email',
        '#attributes' => array('placeholder' => t('Email Address')),
      ];

      $form['dedication-information']['recipient_first_name'] = array(
        '#type' => 'textfield',
        '#prefix' => '<div class="data-dedication-info">',
        '#title' => t("Recipient's Name "),
        '#maxlength' => '30',
        '#attributes' => array('placeholder' => t('First Name')),
      );

      $form['dedication-information']['recipient_last_name'] = array(
        '#type' => 'textfield',
        '#suffix' => '</div>',
        '#title' => t("Recipient's Last Name "),
        '#maxlength' => '30',
        '#attributes' => array('placeholder' => t('Last Name')),
      );

      // Textarea.
      $form['dedication-information']['recipient_message'] = [
        '#type' => 'textarea',
        '#title' => t('Message to the Recipient'),
        '#description' => t('250 Character Limit.'),
        '#attributes' => [
          'maxlength' => ['250']
        ],
        '#default_value' => t('I am making this donation in honor of....'),
      ];

      $form['donor-information'] = [
        '#type' => 'fieldset',
        '#title' => $this->t('Your information'),
      ];

      // Select.
      $form['donor-information']['title'] = [
        '#type' => 'select',
        '#title' => $this->t('Title'),
        '#options' => [
          'mr' => $this->t('Mr.'),
          'ms' => $this->t('Ms.'),
          'mrs' => $this->t('Mrs.'),
          'miss' => $this->t('Miss'),
          'dr' => $this->t('Dr.'),
        ],
        '#required' => TRUE,
        '#empty_option' => $this->t('Please select'),
      ];

      $form['donor-information']['first_name'] = array(
        '#type' => 'textfield',
        '#title' => t('First Name '),
        '#maxlength' => '29',
        '#required' => TRUE,
      );

      $form['donor-information']['last_name'] = array(
        '#type' => 'textfield',
        '#title' => t('Last Name '),
        '#maxlength' => '29',
        '#required' => TRUE,
      );

      $form['donor-information']['email'] = [
        '#type' => 'email',
        '#title' => $this->t('Email'),
        '#maxlength' => '60',
        '#required' => TRUE,
        '#field_suffix' => $this->t('Your receipt will be emailed here.'),
      ];

      $form['donor-information']['comment'] = array(
        '#type' => 'textfield',
        '#title' => t('Leave a comment'),
        '#field_suffix' => $this->t('Max. 100 characters .'),
        '#maxlength' => '100'
      );

      $form['payment-details'] = [
        '#type' => 'fieldset',
        '#title' => $this->t('Payment Details'),
      ];

      $form['payment-details']['name'] = array(
        '#type' => 'textfield',
        '#title' => t('Name'),
        '#maxlength' => '50',
        '#required' => TRUE,
      );

      // Stripe field.
      $form['payment-details']['stripe'] = [
        '#type' => 'stripe',
        '#title' => $this->t('Credit card'),
        '#required' => TRUE,
        // The selectors are gonna be looked within the enclosing form only.
        "#stripe_selectors" => [
        ]
      ];

      $form['payment-details']['address1'] = array(
        '#type' => 'textfield',
        '#title' => t('Address'),
        '#maxlength' => '50',
        '#required' => TRUE,
      );

      $form['payment-details']['address2'] = array(
        '#type' => 'textfield',
        '#title' => t('Address Line 2'),
        '#maxlength' => '50',
      );

      $form['payment-details']['city'] = array(
        '#type' => 'textfield',
        '#title' => t('City'),
        '#maxlength' => '50',
        '#required' => TRUE,
      );

      $form['payment-details']['country'] = [
        '#type' => 'select',
        '#title' => $this->t('Country'),
        '#maxlength' => '50',
        '#options' => $countries,
        '#required' => TRUE,
        '#empty_option' => $this->t('Please select'),
      ];

      $form['payment-details']['zip'] = array(
        '#type' => 'textfield',
        '#title' => t('Postal code'),
        '#maxlength' => '12',
        '#required' => TRUE,
      );

      $form['actions']['#type'] = 'actions';
      $form['actions']['submit'] = array(
        '#type' => 'submit',
        '#value' => $this->t('Send your gift'),
        '#button_type' => 'primary',
      );
      return $form;
    }
    else {
      drupal_set_message(t('Campaign complete.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);

    $query = db_select('campaign_field_data', 'c')
      ->condition('id', $form_state->getValue('campaign_status'), '=')
      ->fields('c', array('campaign_status'));

    $result = $query->execute();
    foreach ($result as $record) {
      $campaignStatus = $record->campaign_status;
    }

    if ($campaignStatus != 'open'){
      $form_state->setErrorByName('campaign_status', $this->t('Sorry! This campaign is now complete.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Get the current user
    $userId = \Drupal::currentUser()->id();

    // Trying things
    $schemeHttpHost = \Drupal::request()->getSchemeAndHttpHost();

    // Get currency from plugin
    $config = \Drupal::config('campaign_kit.settings');
    $clientPlugin =  $config->get('campaign_kit');

    /**
     *  Stripe settings
     */
    if ($this->checkTestStripeApiKey()) {
      // Make test charge if we have test environment and api key.
      // Token is created using checkout or form element!
      // Get the payment token ID submitted by the form:
      $stripeToken = $form_state->getValue('stripe');
      $currency = $form_state->getValue('currency_code');
      $campaignId = $form_state->getValue('item_number');
      $campaignTitle = $form_state->getValue('test_item');
      $email = $form_state->getValue('email');
      // Create charge at Stripe
      $charge = $this->createCharge($stripeToken, $form_state->getValue('amount'), $currency, $campaignId, $campaignTitle, $email);

      // Save Campaign Donation, Campaign Transaction, and Payment entities.
      if ($charge->status == 'succeeded') {
        // Create Campaign Donation entity
        $campaignDonation = CampaignDonation::create(
          array(
            'type' => 'campaign_donation',
            'name' => $form_state->getValue('first_name').' '.$form_state->getValue('last_name'),
            'user_id' => array(
              'target_id' => $userId,
            ),
            'payment_status' => 'Confirmed',
            'anonymous' => 1,
            'first_name' => $form_state->getValue('first_name'),
            'last_name' => $form_state->getValue('last_name'),
            'mail' => $form_state->getValue('email'),
            'amount' => $form_state->getValue('amount'),
            'campaign_id' => array(
              'target_id' => $form_state->getValue('item_number'),
            ),
            'payment_txn_id' => $charge->id,
            'currency' => $form_state->getValue('currency_code'),
            'referred_by' => '',
            'title' => $form_state->getValue('title'),
            'opt_in' => 0,
            'payment_processor' => 'payment_processor',
            'dedicate_donation' => $form_state->getValue('dedicate_donation'),
            'dedicate_type' => $form_state->getValue('dedicate_type'),
            'honoree_first_name' => $form_state->getValue('honoree_first_name'),
            'honoree_last_name' => $form_state->getValue('honoree_last_name'),
            'recipient_mail' => $form_state->getValue('recipient_mail'),
            'recipient_first_name' => $form_state->getValue('recipient_first_name'),
            'recipient_last_name' => $form_state->getValue('recipient_last_name'),
            'recipient_message' => array(
              'value' => $form_state->getValue('recipient_message'),
              'format' => 'basic_html'
            ),
            'donation_frequency' => $form_state->getValue('time'),
          )
        );
        $campaignDonation->save();
        $campaignDonationId = $campaignDonation->id();

        // Create Campaign Transaction entity
        $campaignTransaction = CampaignTransaction::create(
          array(
            'type' => 'campaign_transaction',
            'user_id' => array(
              'target_id' => $userId,
            ),
            'name' => $form_state->getValue('name'),
            'address_1' => $form_state->getValue('address1'),
            'address_2' => $form_state->getValue('address2'),
            'city' => $form_state->getValue('city'),
            'country' => $form_state->getValue('country'),
            'postal_code' => $form_state->getValue('zip'),
            'campaign_id' => array(
              'target_id' => $form_state->getValue('item_number'),
            ),
          )
        );
        $campaignTransaction->save();

        // Saving a Payment
        // You need to pass the bundle when you create a payment.
        $payment = Payment::create([
          'bundle' => 'campaign',
          //'payment_id' => $charge->id,
          //'campaign_id' => 1,
        ]);

        // Set up my custom payment method.
        $payment_method_manager = \Drupal::service('plugin.manager.payment.method');
        $payment->setPaymentMethod($payment_method_manager->createInstance($this->campaign_service->getPaymentMethodActivated()));

        // Set custom payment type
        $payment_type_manager = \Drupal::service('plugin.manager.payment.type');
        $payment->setPaymentType($payment_type_manager->createInstance($this->campaign_service->getPaymentTypeActivated()));

        // Viewing the payment page would cause a fatal error in  \Drupal\payment\Plugin\views\field\PaymentAmount if not set.
        // Also make sure you have imported/enabled the currency via the UI or it will be saved as null.
        $payment->setCurrencyCode($form_state->getValue('currency_code'));

        // Create a Line Item instance.
        $line_item_manager = \Drupal::service('plugin.manager.payment.line_item');
        $line_item = $line_item_manager->createInstance('payment_basic')
          ->setDescription($form_state->getValue('test_item'))
          ->setQuantity(1)
          ->setAmount($form_state->getValue('amount'))
          ->setName($form_state->getValue('test_item'))
          ->setCurrencyCode($form_state->getValue('currency_code'));
        $payment->setLineItems([$line_item]);

        $payment->execute();

        // Fires Succeed event.
        $chargeSucceeded = new StripeWebhookChargeSucceeded($charge);

        $this->event_dispatcher->dispatch(StripeEvents::WEBHOOK_CHARGE_SUCCEEDED, $chargeSucceeded);

        // Redirect user
        if ($form_state->getValue('campaign_type') == 'standalone' || $form_state->getValue('campaign_type') == 'parent') {
          // Redirect to thank you page
          $form_state->setRedirect('campaign_kit.campaign_kit_controller_thank_you_id_campaign', array('campaign' => $form_state->getValue('item_number'), 'campaignDonationId' => $campaignDonationId));
        } else {
          // Redirect to child thank you page
          $form_state->setRedirect('campaign_kit.campaign_kit_controller_thank_you_child_id_campaign', array('campaignParentId' => $form_state->getValue('parent_id'), 'campaignChildId' => $form_state->getValue('campaign_status'), 'campaignDonationId' => $campaignDonationId));
        }

      } elseif ($charge->status == 'failed') {
        // TODO: Fires Failed event, re think both.
        $chargeFailed = new StripeWebhookChargeFailed($charge);
        $this->event_dispatcher->dispatch(StripeEvents::WEBHOOK_CHARGE_SUCCEEDED, $chargeFailed);

        drupal_set_message(t("Payment failed. Please contact the site owner."));
      }

      // TODO: Campaign Kit after saving tha donation and when Stripe Payment sends Charge:succeed or Charge:failed, we need to update DB and show a message.

    } else {
      drupal_set_message(t("Please add and configure a payment plugin."), 'error');
    }
  }

  /**
   * Helper function for checking Stripe Api Keys.
   */
  private function checkTestStripeApiKey() {
    $status = FALSE;
    $config = \Drupal::config('payment_stripe.settings');
    if ($config->get('environment') == 'test' && $config->get('apikey.test.secret')) {
      $status = TRUE;
    }
    return $status;
  }

  /**
   * Helper function for test charge.
   *
   * @param string $stripe_token
   *   Stripe API token.
   * @param int $amount
   *   Amount for charge.
   *
   * @return /Stripe/Charge
   *   Charge object.
   */
  private function createCharge($token, $amount, $currency, $campaignId, $campaignTitle, $email) {
    $config = \Drupal::config('payment_stripe.settings');
    $apiKey = $config->get('apikey.test.secret');
    // Set your secret key: remember to change this to your live secret key in production
    // See your keys here: https://dashboard.stripe.com/account/apikeys
    \Stripe\Stripe::setApiKey($apiKey);

    $intAmount = (integer)$amount;
    // Charge the user's card:
    $charge = \Stripe\Charge::create(array(
      "amount" => $intAmount * 100,
      "currency" => $currency,
      "description" => $campaignTitle,
      "source" => $token,
      "metadata" => array(
        "order_id" => $campaignId,
        "campaign_id" => $campaignId,
        "campaign_label" => $campaignTitle,
      ),
      "receipt_email" => $email,
    ));

    return $charge;
  }

}
